package com.njyunzhi.common.util.aliyun.oss;


import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.common.auth.CredentialsProvider;
import com.aliyun.oss.common.auth.DefaultCredentialProvider;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.aliyun.oss.model.MatchMode;
import com.aliyun.oss.model.PolicyConditions;
import com.njyunzhi.common.util.aliyun.AliyunConfig;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;

@Component
public class OssUtil {

    AliyunConfig aliyunConfig;
    OssConfig ossConfig;
    CredentialsProvider credentialsProvider;
    Boolean enable;

    @Autowired
    public OssUtil(AliyunConfig aliyunConfig, OssConfig ossConfig) {
        enable = !(null == aliyunConfig || !aliyunConfig.getEnable() || null == ossConfig || !ossConfig.getEnable());
        if (!enable) {
            return;
        }

        this.aliyunConfig = aliyunConfig;
        this.ossConfig = ossConfig;

        credentialsProvider = new DefaultCredentialProvider(aliyunConfig.getAccessKeyId(), aliyunConfig.getAccessKeySecret());
    }

    /**
     * 获取 WEB 直传 用到的参数
     * @return
     * @throws IOException
     */
    public PolicyResponse getPolicy() throws IOException {
        if (!enable) {
            throw new IOException("请启用 OSS 设置");
        }

        LocalDateTime now = LocalDateTime.now();
        // 上传目录按月份
        String dir = now.format(DateTimeFormatter.ofPattern("yyyyMM")) + "/";
        // 过期时间
        long expireEndTime = System.currentTimeMillis() + ossConfig.getExpireTime() * 1000;
        Date expireDate = new Date(expireEndTime);

        // PostObject请求最大可支持的文件大小为5 GB，即CONTENT_LENGTH_RANGE为5*1024*1024*1024。
        PolicyConditions policyConds = new PolicyConditions();
        policyConds.addConditionItem(PolicyConditions.COND_CONTENT_LENGTH_RANGE, 0, 1048576000);
        policyConds.addConditionItem(MatchMode.StartWith, PolicyConditions.COND_KEY, dir);

        OSS ossClient = new OSSClientBuilder().build(ossConfig.getEndpoint(), credentialsProvider);
        String postPolicy = ossClient.generatePostPolicy(expireDate, policyConds);
        byte[] binaryData = postPolicy.getBytes(StandardCharsets.UTF_8);
        String policy = BinaryUtil.toBase64String(binaryData);
        String signature = ossClient.calculatePostSignature(postPolicy);

        PolicyResponse res = new PolicyResponse();
        res.setAccessId(credentialsProvider.getCredentials().getAccessKeyId());
        res.setPolicy(policy);
        res.setSignature(signature);
        res.setDir(dir);
        res.setHost(ossConfig.getHost());
        res.setExpire(expireEndTime);
        res.setCustomDomain(ossConfig.getCustomDomain());

        return res;
    }

    @Data
    public static class PolicyResponse {
        String accessId;
        String policy;
        String signature;
        String dir;
        String host;
        Long expire;
        String customDomain;
    }
}