package com.njyunzhi.platform.controller;

import cn.dev33.satoken.util.SaResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.common.util.StringUtil;
import com.njyunzhi.framework.accesslimit.AccessLimit;
import com.njyunzhi.framework.log.OpLog;
import com.njyunzhi.platform.common.Constants;
import com.njyunzhi.platform.service.FxTenantService;
import com.njyunzhi.repository.BaseController;
import com.njyunzhi.repository.entity.FxTenant;
import com.njyunzhi.repository.entity.FxUser;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 租户表;(fx_tenant)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2023-10-25
 */
@Api(tags = "租户表对象功能接口")
@RestController
@RequestMapping("/")
public class FxTenantController extends BaseController {

    @Autowired
    private FxTenantService fxTenantService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @OpLog(module = "租户表", action = "查询单条数据", desc = "ID：{{#id}}")
    @AccessLimit()
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/fxTenant/{id}")
    public SaResult queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return SaResult.data(fxTenantService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @OpLog(module = "租户表", action = "分页列表查询")
    @AccessLimit()
    @ApiOperation("分页查询")
    @GetMapping("/fxTenant")
    public SaResult list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                         @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                         @ApiParam("正序排列") @RequestParam(value = "sortAsc", required = false) String sortAsc,
                         @ApiParam("倒序排列") @RequestParam(value = "sortDesc", defaultValue = "created_at") String sortDesc,

                         @ApiParam("开始时间") @RequestParam(value = "startTime", required = false) String startTime,
                         @ApiParam("结束时间") @RequestParam(value = "endTime", required = false) String endTime,
                         @ApiParam("手机号") @RequestParam(value = "phone", required = false) String phone,
                         @ApiParam("租户名称") @RequestParam(value = "tenantName", required = false) String tenantName,
                         @ApiParam("租户编号") @RequestParam(value = "tenantCode", required = false) String tenantCode,
                         @ApiParam("租户状态") @RequestParam(value = "status", required = false) String status


    ) throws Exception {

        FxUser fxUser = (FxUser) currentUser();

        IPage<FxTenant> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<FxTenant> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("created_by", fxUser.getUserId());
        queryWrapper.ge(!StringUtil.isEmpty(startTime), "start_time", startTime).le(!StringUtil.isEmpty(endTime), "end_time", endTime);
        queryWrapper.like(!StringUtil.isEmpty(tenantName), "tenant_name", tenantName);
        queryWrapper.like(!StringUtil.isEmpty(tenantCode), "tenant_code", tenantCode);
        queryWrapper.like(!StringUtil.isEmpty(phone), "phone", phone);
        queryWrapper.eq(!StringUtil.isEmpty(status), "status", status);

        queryWrapper.orderByAsc(StringUtil.isNotEmpty(sortAsc), StringUtil.humpToLine(sortAsc));
        queryWrapper.orderByDesc(StringUtil.isNotEmpty(sortDesc), StringUtil.humpToLine(sortDesc));


        IPage<FxTenant> result = fxTenantService.page(pg, queryWrapper);

        return SaResult.data(result);
    }

    /**
     * 新增数据
     *
     * @param fxTenant 实例对象
     * @return 实例对象
     */
    @OpLog(module = "租户表", action = "新增数据")
    @AccessLimit()
    @ApiOperation("新增数据")
    @PostMapping("/fxTenant")
    public SaResult add(@ApiParam("对象实体") @Validated @RequestBody FxTenant fxTenant) throws Exception {
        //
        long cnt = fxTenantService.countBy("tenant_name", fxTenant.getTenantName());
        if (cnt > 0) {
            return SaResult.error("租户名称重复");
        }

        cnt = fxTenantService.countBy("phone", fxTenant.getPhone());
        if (cnt > 0) {
            return SaResult.error("手机号码重复");
        }

        fxTenantService.createNew(fxTenant);
        return SaResult.data(fxTenant);
    }

    /**
     * 更新数据
     *
     * @param fxTenant 实例对象
     * @return 实例对象
     */
    @OpLog(module = "租户表", action = "更新数据")
    @AccessLimit()
    @ApiOperation("更新数据")
    @PutMapping("/fxTenant/{id}")
    public SaResult edit(@ApiParam("对象实体") @Validated @RequestBody FxTenant fxTenant,
                         @ApiParam("对象ID") @PathVariable String id) throws Exception {
        //更新自己本表
        if (null == fxTenant.getStatus()) {
            fxTenant.setStatus(Constants.STATUS_READY);
        }
        fxTenant.setTenantId(id);
        fxTenantService.updateById(fxTenant);
        FxTenant fxTenant1 = fxTenantService.getById(id);

        fxTenantService.upDateNew(fxTenant1);

        return SaResult.data(fxTenant);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @OpLog(module = "租户表", action = "删除数据", desc = "ID：{{#id}}")
    @AccessLimit()
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/fxTenant/{id}")
    public SaResult deleteById(@ApiParam("对象ID") @PathVariable String id) {

        fxTenantService.removeSysFxById(id);
        return SaResult.data("success");
    }
}